@@ -1,13 +1,13 @@
 ## Should respect the input class except when there is a conflict or always
-## generate a data frame? 
+## generate a data frame?
 
 ppMethods <- c("BoxCox", "YeoJohnson", "expoTrans", "invHyperbolicSine",
-               "center", "scale", "range", 
-               "knnImpute", "bagImpute", "medianImpute", 
-               "pca", "ica", 
-               "spatialSign", 
-               "ignore", "keep", 
-               "remove", 
+               "center", "scale", "range",
+               "knnImpute", "bagImpute", "medianImpute",
+               "pca", "ica",
+               "spatialSign",
+               "ignore", "keep",
+               "remove",
                "zv", "nzv", "conditionalX",
                "corr")
 
@@ -22,14 +22,14 @@ getRangeBounds <- function(pp) {
 }
 
 #' Pre-Processing of Predictors
-#' 
+#'
 #' Pre-processing transformation (centering, scaling etc.) can be estimated
 #' from the training data and applied to any data set with the same variables.
-#' 
+#'
 #' In all cases, transformations and operations are estimated using the data in
 #' \code{x} and these operations are applied to new data using these values;
 #' nothing is recomputed when using the \code{predict} function.
-#' 
+#'
 #' The Box-Cox (\code{method = "BoxCox"}), Yeo-Johnson (\code{method =
 #' "YeoJohnson"}), and exponential transformations (\code{method =
 #' "expoTrans"})have been "repurposed" here: they are being used to transform
@@ -43,44 +43,44 @@ getRangeBounds <- function(pp) {
 #' the Box-Cox transformation must be strictly positive.) The exponential
 #' transformation of Manly (1976) can also be used for positive or negative
 #' data.
-#' 
+#'
 #' \code{method = "center"} subtracts the mean of the predictor's data (again
 #' from the data in \code{x}) from the predictor values while \code{method =
 #' "scale"} divides by the standard deviation.
-#' 
+#'
 #' The "range" transformation scales the data to be within \code{rangeBounds}. If new
 #' samples have values larger or smaller than those in the training set, values
 #' will be outside of this range.
-#' 
+#'
 #' Predictors that are not numeric are ignored in the calculations.
-#' 
+#'
 #' \code{method = "zv"} identifies numeric predictor columns with a single
 #' value (i.e. having zero variance) and excludes them from further
 #' calculations. Similarly, \code{method = "nzv"} does the same by applying
 #' \code{\link{nearZeroVar}} exclude "near zero-variance" predictors. The options
-#' \code{freqCut} and \code{uniqueCut} can be used to modify the filter. 
+#' \code{freqCut} and \code{uniqueCut} can be used to modify the filter.
 #'
 #' \code{method = "corr"} seeks to filter out highly correlated predictors. See
-#' \code{\link{findCorrelation}}. 
-#' 
+#' \code{\link{findCorrelation}}.
+#'
 #' For classification, \code{method = "conditionalX"} examines the distribution
 #' of each predictor conditional on the outcome. If there is only one unique
 #' value within any class, the predictor is excluded from further calculations
 #' (see \code{\link{checkConditionalX}} for an example). When \code{outcome} is
 #' not a factor, this calculation is not executed. This operation can be time
 #' consuming when used within resampling via \code{\link{train}}.
-#' 
+#'
 #' The operations are applied in this order: zero-variance filter, near-zero
 #' variance filter, correlation filter, Box-Cox/Yeo-Johnson/exponential transformation, centering,
 #' scaling, range, imputation, PCA, ICA then spatial sign. This is a departure
 #' from versions of \pkg{caret} prior to version 4.76 (where imputation was
 #' done first) and is not backwards compatible if bagging was used for
 #' imputation.
-#' 
+#'
 #' If PCA is requested but centering and scaling are not, the values will still
 #' be centered and scaled. Similarly, when ICA is requested, the data are
 #' automatically centered and scaled.
-#' 
+#'
 #' k-nearest neighbor imputation is carried out by finding the k closest
 #' samples (Euclidian distance) in the training set. Imputation via bagging
 #' fits a bagged tree model for each predictor (as a function of all the
@@ -89,20 +89,20 @@ getRangeBounds <- function(pp) {
 #' of each predictor in the training set, and uses them to fill missing values.
 #' This method is simple, fast, and accepts missing values, but treats each
 #' predictor independently, and may be inaccurate.
-#' 
+#'
 #' A warning is thrown if both PCA and ICA are requested. ICA, as implemented
 #' by the \code{\link[fastICA]{fastICA}} package automatically does a PCA
 #' decomposition prior to finding the ICA scores.
-#' 
+#'
 #' The function will throw an error of any numeric variables in \code{x} has
 #' less than two unique values unless either \code{method = "zv"} or
 #' \code{method = "nzv"} are invoked.
-#' 
+#'
 #' Non-numeric data will not be pre-processed and there values will be in the
 #' data frame produced by the \code{predict} function. Note that when PCA or
 #' ICA is used, the non-numeric columns may be in different positions when
 #' predicted.
-#' 
+#'
 #' @aliases preProcess preProcess.default predict.preProcess
 #' @param x a matrix or data frame. Non-numeric predictors are allowed but will
 #' be ignored.
@@ -130,12 +130,12 @@ getRangeBounds <- function(pp) {
 #' @param numUnique how many unique values should \code{y} have to estimate the
 #' Box-Cox transformation?
 #' @param verbose a logical: prints a log as the computations proceed
-#' @param freqCut the cutoff for the ratio of the most common value to the 
+#' @param freqCut the cutoff for the ratio of the most common value to the
 #' second most common value. See \code{\link{nearZeroVar}}.
-#' @param uniqueCut the cutoff for the percentage of distinct values out of 
-#' the number of total samples. See \code{\link{nearZeroVar}}. 
-#' @param cutoff a numeric value for the pair-wise absolute correlation cutoff. 
-#' See \code{\link{findCorrelation}}.  
+#' @param uniqueCut the cutoff for the percentage of distinct values out of
+#' the number of total samples. See \code{\link{nearZeroVar}}.
+#' @param cutoff a numeric value for the pair-wise absolute correlation cutoff.
+#' See \code{\link{findCorrelation}}.
 #' @param rangeBounds a two-element numeric vector specifying closed interval
 #' for range transformation
 #' @param \dots additional arguments to pass to \code{\link[fastICA]{fastICA}},
@@ -146,51 +146,51 @@ getRangeBounds <- function(pp) {
 #' transformation values, see \code{\link{BoxCoxTrans}}} \item{mean}{a vector
 #' of means (if centering was requested)} \item{std}{a vector of standard
 #' deviations (if scaling or PCA was requested)} \item{rotation}{a matrix of
-#' eigenvectors if PCA was requested} \item{method}{the value of\code{method}}
-#' \item{thresh}{the value of\code{thresh}} \item{ranges}{a matrix of min and
+#' eigenvectors if PCA was requested} \item{method}{the value of \code{method}}
+#' \item{thresh}{the value of \code{thresh}} \item{ranges}{a matrix of min and
 #' max values for each predictor when \code{method} includes "range" (and
 #' \code{NULL} otherwise)} \item{numComp}{the number of principal components
 #' required of capture the specified amount of variance} \item{ica}{contains
 #' values for the \code{W} and \code{K} matrix of the decomposition}
 #' \item{median}{a vector of medians (if median imputation was requested)}
-#' 
+#'
 #' \code{predict.preProcess} will produce a data frame.
 #' @author Max Kuhn, median imputation by Zachary Mayer
 #' @seealso \code{\link{BoxCoxTrans}}, \code{\link{expoTrans}}
 #' \code{\link[MASS]{boxcox}}, \code{\link[stats]{prcomp}},
 #' \code{\link[fastICA]{fastICA}}, \code{\link{spatialSign}}
 #' @references \url{http://topepo.github.io/caret/pre-processing.html}
-#' 
+#'
 #' Kuhn and Johnson (2013), Applied Predictive Modeling, Springer, New York
 #' (chapter 4)
-#' 
+#'
 #' Kuhn (2008), Building predictive models in R using the caret
 #' (\url{http://www.jstatsoft.org/article/view/v028i05/v28i05.pdf})
-#' 
+#'
 #' Box, G. E. P. and Cox, D. R. (1964) An analysis of transformations (with
 #' discussion). Journal of the Royal Statistical Society B, 26, 211-252.
-#' 
+#'
 #' Box, G. E. P. and Tidwell, P. W. (1962) Transformation of the independent
 #' variables. Technometrics 4, 531-550.
-#' 
+#'
 #' Manly, B. L. (1976) Exponential data transformations. The Statistician, 25,
 #' 37 - 42.
-#' 
+#'
 #' Yeo, I-K. and Johnson, R. (2000). A new family of power transformations to
 #' improve normality or symmetry. Biometrika, 87, 954-959.
 #' @keywords utilities
 #' @examples
-#' 
+#'
 #' data(BloodBrain)
 #' # one variable has one unique value
 #' \dontrun{
 #' preProc <- preProcess(bbbDescr)
-#' 
+#'
 #' preProc  <- preProcess(bbbDescr[1:100,-3])
 #' training <- predict(preProc, bbbDescr[1:100,-3])
 #' test     <- predict(preProc, bbbDescr[101:208,-3])
 #' }
-#' 
+#'
 #' @export preProcess
 preProcess <- function(x, ...) UseMethod("preProcess")
 
@@ -206,8 +206,8 @@ preProcess.default <- function(x, method = c("center", "scale"),
                                outcome = NULL,
                                fudge = .2,
                                numUnique = 3,
-                               verbose = FALSE, 
-                               freqCut = 95/5, 
+                               verbose = FALSE,
+                               freqCut = 95/5,
                                uniqueCut = 10,
                                cutoff = 0.9,
                                rangeBounds = c(0, 1),
@@ -218,7 +218,13 @@ preProcess.default <- function(x, method = c("center", "scale"),
   tmp <- pre_process_options(method, column_types)
   method <- tmp$opts
   wildcards <- tmp$wildcards
-  
+
+  if (any(method == "corr") & !any(method == "zv")) {
+    method <- unique(c(method, "zv"))
+    if (verbose)
+      cat("A zero-variance filter was added for the correlation filter\n")
+  }
+
   ## the row.norm option in fastICA states: "logical value indicating whether rows
   ## of the data matrix X should be standardized beforehand." Basically, this means that
   ## we would center *and* scale before the ICA step, so let's adjust the "scale" method too
@@ -226,7 +232,7 @@ preProcess.default <- function(x, method = c("center", "scale"),
     theDots <- list(...)
     row.norm <- if(is.null(list(...)$row.norm)) FALSE else list(...)$row.norm
   }
-  
+
   ## check for zero-variance predictors
   if(any(names(method) == "zv")){
     if(is.data.frame(x)) {
@@ -244,7 +250,7 @@ preProcess.default <- function(x, method = c("center", "scale"),
   }
   ## check for near-zero-variance predictors
   if(any(names(method) == "nzv")){
-    is_nzv <- nearZeroVar(x[, !(colnames(x) %in% method$ignore), drop = FALSE], 
+    is_nzv <- nearZeroVar(x[, !(colnames(x) %in% method$ignore), drop = FALSE],
                           freqCut = freqCut, uniqueCut = uniqueCut)
     if(length(is_nzv) > 0) {
       removed <- colnames(x[, !(colnames(x) %in% method$ignore), drop = FALSE])[is_nzv]
@@ -253,8 +259,8 @@ preProcess.default <- function(x, method = c("center", "scale"),
       if(verbose) cat(paste(" ", length(removed), "near-zero variance predictors were removed.\n"))
     }
     method$nzv <- NULL
-  }  
-  ##  check the distribution of the columns of x conditioned on the levels of y and 
+  }
+  ##  check the distribution of the columns of x conditioned on the levels of y and
   ## identifies columns of x that are sparse within groups of y
   if(any(names(method) == "conditionalX") & is.factor(outcome)){
     bad_pred <- checkConditionalX(x = x[, !(colnames(x) %in% method$ignore), drop = FALSE],
@@ -266,11 +272,11 @@ preProcess.default <- function(x, method = c("center", "scale"),
       if(verbose) cat(paste(" ", length(removed), "conditionally zero variance predictors.\n"))
     }
     method$conditionalX <- NULL
-  }    
-  
+  }
+
   ## check for highly correlated predictors
   if(any(names(method) == "corr")){
-    cmat <- try(cor(x[, !(colnames(x) %in% method$ignore), drop = FALSE], 
+    cmat <- try(cor(x[, !(colnames(x) %in% c(method$ignore, method$remove)), drop = FALSE], 
                     use = "pairwise.complete.obs"), 
                 silent = TRUE)
     if(class(cmat)[1] != "try-error") {
@@ -283,21 +289,21 @@ preProcess.default <- function(x, method = c("center", "scale"),
     }
     method$corr <- NULL
   }
-  
-  x <- x[, !(colnames(x) %in% method$remove), drop = FALSE] 
-  method = sapply(names(method), function(u) 
+
+  x <- x[, !(colnames(x) %in% method$remove), drop = FALSE]
+  method = sapply(names(method), function(u)
     if(u != 'remove'){
       method[[u]][ which(( method[[u]] %in% colnames(x)))]
-    } else { 
+    } else {
       method[[u]]
     }, simplify = FALSE
   )
-  
+
   if(any(names(method) == "invHyperbolicSine")) {
     if(verbose) cat(" applying invHyperbolicSine\n")
     for(i in method$invHyperbolicSine) x[,i] <- invHyperbolicSineFunc(x[,i])
-  } 
-  
+  }
+
   if(any(names(method) == "BoxCox")) {
     bc <- group_bc(x[, method$BoxCox, drop = FALSE],
                    fudge = fudge,
@@ -310,15 +316,15 @@ preProcess.default <- function(x, method = c("center", "scale"),
     }
     for(i in method$BoxCox) x[,i] <- predict(bc[[i]], x[,i])
   } else bc <- NULL
-  
-  
+
+
   if(any(names(method) == "YeoJohnson")) {
     yj <- vapply(
       x[, method$YeoJohnson, drop = FALSE],
       recipes::estimate_yj,
       c(lambda = 0),
       limits = c(-3, 3), ## consistent with `car` defaults
-      nunique = numUnique
+      num_unique = numUnique
     )
     yj <- yj[!is.null(yj) & !is.na(yj)]
     if(length(yj) > 0) {
@@ -337,14 +343,14 @@ preProcess.default <- function(x, method = c("center", "scale"),
       if(verbose) cat(" all of the transformations failed\n")
       yj <- NULL
     }
-  } else yj <- NULL  
+  } else yj <- NULL
 
   if(any(names(method) == "expoTrans")) {
-    if(verbose) 
-      cat("Estimating exponential transformations for", 
+    if(verbose)
+      cat("Estimating exponential transformations for",
           length(method$expoTrans), "predictors...")
     if(is.data.frame(x)) {
-      et <- lapply(x[, method$expoTrans, drop = FALSE], 
+      et <- lapply(x[, method$expoTrans, drop = FALSE],
                    expoTrans.default, numUnique = numUnique)
     } else {
       et <- apply(x[, method$expoTrans, drop = FALSE], 2,
@@ -365,13 +371,13 @@ preProcess.default <- function(x, method = c("center", "scale"),
               immediate. = TRUE)
       et <- et[!(names(et) %in% omit_expo)]
     }
-  } else et <- NULL  
+  } else et <- NULL
   if(any(names(method)  %in% c("center"))) {
     if(verbose) cat("Calculating", length(method$center), "means for centering\n")
-    centerValue <- apply(x[, method$center, drop = FALSE], 2, mean, na.rm = na.remove) 
+    centerValue <- apply(x[, method$center, drop = FALSE], 2, mean, na.rm = na.remove)
     x[, method$center] <- sweep(x[, method$center, drop = FALSE], 2, centerValue, "-")
   } else centerValue <- NULL
-  
+
   if(any(names(method) %in% c("scale"))) {
     if(verbose) cat("Calculating", length(method$scale), "standard deviations for scaling\n")
     scaleValue <- apply(x[, method$scale, drop = FALSE], 2, sd, na.rm = na.remove)
@@ -381,16 +387,16 @@ preProcess.default <- function(x, method = c("center", "scale"),
       warning(wrn, immediate. = TRUE)
       scaleValue[which(is.na(scaleValue))] <- 1
     }
-    
+
     if(any(scaleValue == 0)){
       wrn <- paste("These variables have zero variances:",
                    paste(names(scaleValue)[which(scaleValue == 0)], collapse = ", "))
       warning(wrn, immediate. = TRUE)
       scaleValue[which(scaleValue == 0)] <- 1
-    }    
+    }
     x[, method$scale] <- sweep(x[, method$scale, drop = FALSE], 2, scaleValue, "/")
   } else scaleValue <- NULL
-  
+
   if(any(names(method) == "range")) {
     if(verbose) cat("Calculating", length(method$range), "statistcs for scaling to a range\n")
     ## check rangeBounds consistency
@@ -399,9 +405,9 @@ preProcess.default <- function(x, method = c("center", "scale"),
     if(rangeBounds[1] >= rangeBounds[2])
       stop("'rangeBounds' interval is empty")
 
-    ranges <- apply(x[, method$range, drop = FALSE], 
-                    2, 
-                    function(x) c(min(x, na.rm = na.remove), 
+    ranges <- apply(x[, method$range, drop = FALSE],
+                    2,
+                    function(x) c(min(x, na.rm = na.remove),
                                   max(x, na.rm = na.remove)))
     ## check for zero variance
     is_same <- apply(ranges, 2, function(x) x[1] == x[2])
@@ -416,7 +422,7 @@ preProcess.default <- function(x, method = c("center", "scale"),
                                (ranges[2,] - ranges[1,]) / (rangeBounds[2] - rangeBounds[1]), "/")
     x[, method$range] <- sweep(x[, method$range, drop = FALSE], 2, rangeBounds[1], "+")
   } else ranges <- NULL
-  
+
   if(any(names(method) == "bagImpute")){
     if(verbose) cat("Computing bagging models for", length(method$bagImpute), "predictors...")
     bagModels <- as.list(method$bagImpute)
@@ -424,14 +430,14 @@ preProcess.default <- function(x, method = c("center", "scale"),
     bagModels <- lapply(bagModels, bagImp, x = x)
     if(verbose) cat(" done\n")
   } else bagModels <- NULL
-  
+
   if (any(names(method) == "medianImpute")) {
-    if(verbose) 
+    if(verbose)
       cat("Computing medians for",
           length(method$medianImpute), "predictors...")
-    medianValue <- apply(x[, method$medianImpute, drop = FALSE], 
+    medianValue <- apply(x[, method$medianImpute, drop = FALSE],
                          2, median, na.rm=TRUE)
-    
+
     if (any(is.na(medianValue))) {
       warning(
         paste(
@@ -442,19 +448,19 @@ preProcess.default <- function(x, method = c("center", "scale"),
     }
     if(verbose) cat(" done\n")
   } else medianValue <- NULL
-  
-  ## TODO This should be moved inside of pca or ica code 
+
+  ## TODO This should be moved inside of pca or ica code
   ## and done after centering and scaling
   x <- x[complete.cases(x),,drop = FALSE]
-  
+
   if(any(names(method) == "pca")) {
-    if(verbose) 
+    if(verbose)
       cat("Computing PCA loadings for",
-          length(method$pca), 
+          length(method$pca),
           "predictors\n")
     tmp <- prcomp(x[, method$pca, drop = FALSE], scale = TRUE, retx = FALSE)
     if(is.null(pcaComp)) {
-      cumVar <- cumsum(tmp$sdev^2/sum(tmp$sdev^2)) 
+      cumVar <- cumsum(tmp$sdev^2/sum(tmp$sdev^2))
       numComp <- max(2, which.max(cumVar >= thresh))
     } else numComp <- min(pcaComp, ncol(tmp$rotation))
     rot <- tmp$rotation[,1:numComp]
@@ -462,14 +468,14 @@ preProcess.default <- function(x, method = c("center", "scale"),
     rot <- NULL
     numComp <- NULL
   }
-  
+
   if(any(names(method) == "ica")) {
-    ## TODO What if range is used? 
-    if(verbose) 
+    ## TODO What if range is used?
+    if(verbose)
       cat("Computing ICA loadings for",
-          length(method$ica), 
+          length(method$ica),
           "predictors\n")
-    requireNamespaceQuietStop("fastICA")    
+    requireNamespaceQuietStop("fastICA")
     tmp <- fastICA::fastICA(x[, method$ica, drop = FALSE], ...)
     ica <- list(row.norm = row.norm,
                 K = tmp$K,
@@ -477,7 +483,7 @@ preProcess.default <- function(x, method = c("center", "scale"),
   } else {
     ica <- NULL
   }
-  
+
   out <- list(dim = c(nrow(x), length(unique(unlist(method)))),
               bc = bc,
               yj = yj,
@@ -497,7 +503,7 @@ preProcess.default <- function(x, method = c("center", "scale"),
               knnSummary = knnSummary,
               bagImp = bagModels,
               median = medianValue,
-              data = if(any(names(method) == "knnImpute")) 
+              data = if(any(names(method) == "knnImpute"))
                 x[complete.cases(x),method$knnImpute,drop = FALSE] else NULL,
               rangeBounds = rangeBounds)
   out <- structure(out, class = "preProcess")
@@ -510,23 +516,23 @@ preProcess.default <- function(x, method = c("center", "scale"),
 predict.preProcess <- function(object, newdata, ...) {
   if(is.vector(object$method) & !is.list(object$method))
     object <- convert_method(object)
-  
+
   dataNames <- colnames(newdata)
   oldClass <- class(newdata)
-  
+
   if(!is.null(object$method$remove)) {
     if(length(object$method$remove) > 0)
       newdata <- newdata[, !(colnames(newdata) %in% object$method$remove), drop = FALSE]
     if(ncol(newdata) == 0)
       stop("All predctors were removed as determined by `preProcess`")
   }
-  
+
   if(!is.null(object$invHyperbolicSine)) {
     for(i in object$invHyperbolicSine) {
       newdata[,i] <- invHyperbolicSineFunc(newdata[,i])
     }
   }
-  
+
   if(!is.null(object$bc)) {
     lam <- unlist(lapply(object$bc, function(x) x$lambda))
     lamIndex <- which(!is.na(lam))
@@ -538,7 +544,7 @@ predict.preProcess <- function(object, newdata, ...) {
       }
     }
   }
-  
+
   if(!is.null(object$yj)) {
     lam <- get_yj_lambda(object$yj)
     lam <- lam[!is.na(lam)]
@@ -548,21 +554,21 @@ predict.preProcess <- function(object, newdata, ...) {
         newdata[,who] <- recipes::yj_trans(newdata[,who], lam[who])
       }
     }
-  } 
-  
+  }
+
   if(!is.null(object$et)) {
     for(i in seq(along = object$et)) {
       who <-  names(object$et)[i]
       newdata[,who] <- predict(object$et[[who]], newdata[,who])
     }
-  }  
-  
+  }
+
   if(any(names(object$method) == "range")) {
     rangeBounds <- getRangeBounds(object)
-    newdata[, object$method$range] <- 
-      sweep(newdata[, object$method$range, drop = FALSE], 2, 
+    newdata[, object$method$range] <-
+      sweep(newdata[, object$method$range, drop = FALSE], 2,
             object$ranges[1,], "-")
-    newdata[, object$method$range] <- 
+    newdata[, object$method$range] <-
       sweep(newdata[, object$method$range, drop = FALSE], 2,
             (object$ranges[2,] - object$ranges[1,]) /
               (rangeBounds$upper - rangeBounds$lower), "/")
@@ -570,17 +576,17 @@ predict.preProcess <- function(object, newdata, ...) {
       sweep(newdata[, object$method$range, drop = FALSE], 2,
             rangeBounds$lower, "+")
   }
-  
-  if(any(names(object$method) == "center")) 
-    newdata[, object$method$center] <- 
+
+  if(any(names(object$method) == "center"))
+    newdata[, object$method$center] <-
     sweep(newdata[, object$method$center, drop = FALSE], 2, object$mean, "-")
-  if(any(names(object$method) %in% c("scale"))) 
-    newdata[, object$method$scale] <- 
+  if(any(names(object$method) %in% c("scale")))
+    newdata[, object$method$scale] <-
     sweep(newdata[, object$method$scale, drop = FALSE], 2, object$std, "/")
-  
+
   cc <- complete.cases(newdata)
   if(any(names(object$method) == "knnImpute") && any(!cc))  {
-    hasMiss <- newdata[!cc,object$method$knnImpute,drop = FALSE]      
+    hasMiss <- newdata[!cc,object$method$knnImpute,drop = FALSE]
     miss_names <- colnames(hasMiss)
     hasMiss <- apply(hasMiss,
                      1,
@@ -599,7 +605,7 @@ predict.preProcess <- function(object, newdata, ...) {
       } else newdata[!cc, object$method$knnImpute] <- as.matrix(hasMiss)
     }
   }
-  
+
   if(any(names(object$method) == "bagImpute") && any(!cc)) {
     requireNamespaceQuietStop("ipred")
     hasMiss <- newdata[!cc,,drop = FALSE]
@@ -612,7 +618,7 @@ predict.preProcess <- function(object, newdata, ...) {
     for(i in seq(along = missingVars)) {
       preds <- predict(object$bagImp[[missingVars[i]]]$model,
                        hasMiss[, !colnames(hasMiss) %in% missingVars[i], drop = FALSE])
-      
+
       hasMiss[is.na(hasMiss[,missingVars[i]]),
               missingVars[i]] <- preds[is.na(hasMiss[,missingVars[i]])]
     }
@@ -624,7 +630,7 @@ predict.preProcess <- function(object, newdata, ...) {
       } else newdata[!cc,] <- as.matrix(hasMiss)
     }
   }
-  
+
   if (any(names(object$method) == "medianImpute") && any(!cc)) {
     missingVars <- apply(newdata, 2, function(x) any(is.na(x)))
     missingVars <- if(is.null(names(missingVars))) which(missingVars) else names(missingVars)[missingVars]
@@ -632,7 +638,7 @@ predict.preProcess <- function(object, newdata, ...) {
       newdata[is.na(newdata[, v]), v] <- object$median[v]
     }
   }
-  
+
   if(any(names(object$method) == "pca")) {
     pca_cols <- newdata[, object$method$pca, drop = FALSE]
     pca_cols <-if(is.matrix(pca_cols)) pca_cols %*% object$rotation else as.matrix(pca_cols) %*% object$rotation
@@ -649,7 +655,7 @@ predict.preProcess <- function(object, newdata, ...) {
     }
     if(length(discard) > 0) newdata <- newdata[, !(colnames(newdata) %in% discard), drop = FALSE]
   }
-  
+
   if(any(names(object$method) == "ica")) {
     ica_cols <- newdata[, object$method$ica, drop = FALSE]
     if(!is.matrix(ica_cols)) ica_cols <- as.matrix(ica_cols)
@@ -668,21 +674,21 @@ predict.preProcess <- function(object, newdata, ...) {
     }
     if(length(discard) > 0) newdata <- newdata[, !(colnames(newdata) %in% discard), drop = FALSE]
   }
-  
+
   wc <- object$wildcards
-  if(any(names(object$method) == "spatialSign") | 
-     any(wc$PCA == "spatialSign") | 
+  if(any(names(object$method) == "spatialSign") |
+     any(wc$PCA == "spatialSign") |
      any(wc$ICA == "spatialSign")){
     ss_col_names <- object$method$spatialSign
     ## adjust for PCA/ICA column wildcards
-    if(length(wc$PCA) > 0 && any(wc$PCA == "spatialSign")) 
+    if(length(wc$PCA) > 0 && any(wc$PCA == "spatialSign"))
       ss_col_names <- c(ss_col_names, colnames(pca_cols))
-    if(length(wc$ICA) > 0 && any(wc$ICA == "spatialSign")) 
+    if(length(wc$ICA) > 0 && any(wc$ICA == "spatialSign"))
       ss_col_names <- c(ss_col_names, colnames(ica_cols))
-    
+
     newdata[, ss_col_names] <- spatialSign(newdata[, ss_col_names, drop = FALSE])
   }
-  
+
   newdata
 }
 
@@ -690,40 +696,40 @@ predict.preProcess <- function(object, newdata, ...) {
 print.preProcess <- function(x, ...) {
   #   printCall(x$call)
   cat("Created from", x$dim[1], "samples and", x$dim[2], "variables\n\n")
-  
-  
+
+
   pp_num <- unlist(lapply(x$method, length))
   if(any(unlist(x$wildcards) == "spatialSign")) {
     if(any(x$wildcards$PCA == "spatialSign"))
-      pp_num["spatialSign"] <- pp_num["spatialSign"] + x$pcaComp 
+      pp_num["spatialSign"] <- pp_num["spatialSign"] + x$pcaComp
     if(any(x$wildcards$ICA == "spatialSign"))
-      pp_num["spatialSign"] <- pp_num["spatialSign"] + x$numComp     
+      pp_num["spatialSign"] <- pp_num["spatialSign"] + x$numComp
   }
-  pp <- paste0("  - ", names(x$method), " (", pp_num, ")\n")  
+  pp <- paste0("  - ", names(x$method), " (", pp_num, ")\n")
   pp <- pp[order(pp)]
   pp <- gsub("invHyperbolicSine", "Inverve Hyperbolic Sine transformation", pp)
-  pp <- gsub("BoxCox", "Box-Cox transformation", pp)  
-  pp <- gsub("YeoJohnson", "Yeo-Johnson transformation", pp)    
-  pp <- gsub("expoTrans", "exponential transformation", pp)   
+  pp <- gsub("BoxCox", "Box-Cox transformation", pp)
+  pp <- gsub("YeoJohnson", "Yeo-Johnson transformation", pp)
+  pp <- gsub("expoTrans", "exponential transformation", pp)
   pp <- gsub("scale", "scaled", pp)
   pp <- gsub("center", "centered", pp)
   pp <- gsub("pca", "principal component signal extraction", pp)
   pp <- gsub("ica", "independent component signal extraction", pp)
   pp <- gsub("spatialSign", "spatial sign transformation", pp)
   pp <- gsub("knnImpute", paste(x$k, "nearest neighbor imputation"), pp)
-  pp <- gsub("bagImpute", "bagged tree imputation", pp)  
-  pp <- gsub("medianImpute", "median imputation", pp)    
+  pp <- gsub("bagImpute", "bagged tree imputation", pp)
+  pp <- gsub("medianImpute", "median imputation", pp)
 
   rangeBounds <- getRangeBounds(x)
   pp <- gsub("range", paste0("re-scaling to [", rangeBounds$lower, ", ", rangeBounds$upper, "]"), pp)
 
-  pp <- gsub("remove", "removed", pp)  
-  pp <- gsub("ignore", "ignored", pp)  
-  
+  pp <- gsub("remove", "removed", pp)
+  pp <- gsub("ignore", "ignored", pp)
+
   cat("Pre-processing:\n")
   cat(pp, sep = "")
   cat("\n")
-  
+
   if(any(names(x$method) == "BoxCox")) {
     cat("Lambda estimates for Box-Cox transformation:\n")
     if(length(x$bc) < 11) {
@@ -734,7 +740,7 @@ print.preProcess <- function(x, ...) {
     } else print(summary(unlist(lapply(x$bc, function(x) x$lambda))))
     cat("\n")
   }
-  
+
   if(any(names(x$method) == "YeoJohnson")) {
     cat("Lambda estimates for Yeo-Johnson transformation:\n")
     lmbda <- get_yj_lambda(x$yj)
@@ -744,7 +750,7 @@ print.preProcess <- function(x, ...) {
       if(naLmbda > 0) cat(" (#NA: ", naLmbda, ")\n", sep = "")
     } else print(summary(lmbda))
     cat("\n")
-  }  
+  }
 
   if(any(names(x$method) == "pca")) {
     if(is.null(x$pcaComp)) {
@@ -754,14 +760,14 @@ print.preProcess <- function(x, ...) {
     } else {
       cat("PCA used", x$pcaComp, ifelse(x$pcaComp > 1, "components", "component"), "as specified")
     }
-    if(length(x$wildcards$PCA) > 0) 
+    if(length(x$wildcards$PCA) > 0)
       cat(" and will be used in the spatial sign transformation")
     cat("\n")
   }
-  
+
   if(any(names(x$method) == "ica")) {
     cat("ICA used", ncol(x$ica$W), "components")
-    if(length(x$wildcards$ICA) > 0) 
+    if(length(x$wildcards$ICA) > 0)
       cat(" and will be used in the spatial sign transformation")
     cat("\n")
   }
@@ -796,7 +802,7 @@ bagImp <- function(var, x, B = 10) {
   mod <- ipred::bagging(as.formula(paste(var, "~.")),
                         data = x,
                         nbagg = B,
-                        x = FALSE, 
+                        x = FALSE,
                         keepX = FALSE)
   trim_code <- getModelInfo("treebag", FALSE)[[1]]$trim
   list(var = var,
@@ -815,28 +821,28 @@ pre_process_options <- function(opts, vars) {
   if(is.vector(opts) & !is.list(opts)) {
     op_list <- vector(mode = "list", length = length(opts))
     names(op_list) <- opts
-    op_list <- lapply(op_list, 
+    op_list <- lapply(op_list,
                       function(x, y) {
                         x <- y
                         x
                       }, y = names(vars))
     opts <- op_list
   }
-  
+
   ## check names of options
   if(!all(names(opts) %in% ppMethods)) {
     others <- names(opts)[!(names(opts) %in% ppMethods)]
     stop((paste("These pre-processing methods are unknown:",
                 paste("'", others, "'", sep = "", collapse = ", "))))
   }
-  
+
   methods <- names(opts)
-  
+
   ## find and store any PCA/ICA wildcards
   tmp <- check_for_wildcards(opts, verbose = FALSE)
   opts <- tmp$opts
   wildcards <- tmp$wildcards
-  
+
   ## check that each predictor is in the data
   all_op_vars <- unique(unlist(opts))
   if(!all(all_op_vars %in% names(vars))) {
@@ -844,7 +850,7 @@ pre_process_options <- function(opts, vars) {
     stop((paste("These fields are not in the data:",
                 paste("'", others, "'", sep = "", collapse = ", "))))
   } ## get fancy and look for dummy variables and write sensible note?
-  
+
   ## check to make sure calcs are on numbers
   num_vars <- names(vars)[vars]
   not_num <- NULL
@@ -861,7 +867,7 @@ pre_process_options <- function(opts, vars) {
   if(length(not_num) > 0) {
     opts$ignore <- unique(c(opts$ignore, not_num))
   }
-  
+
   ## check for group trans on single predictors
   if("pca" %in% methods && length(opts[["pca"]]) == 1){
     warning(paste("PCA is a group transformation and only a single predictor",
@@ -869,24 +875,24 @@ pre_process_options <- function(opts, vars) {
             immediate. = TRUE)
     opts[["pca"]] <- NULL
   }
-  
-  
+
+
   if("ica" %in% methods && length(opts[["ica"]]) == 1){
     warning(paste("ICA is a group transformation and only a single predictor",
                   "is listed. This method is eliminated."),
             immediate. = TRUE)
     opts[["ica"]] <- NULL
   }
-  
-  if(all(unlist(wildcards) != "spatialSign") & 
-     "spatialSign" %in% methods & 
+
+  if(all(unlist(wildcards) != "spatialSign") &
+     "spatialSign" %in% methods &
      length(opts[["spatialSign"]]) == 1 ){
     warning(paste("Spatial sign is a group transformation and only a single predictor",
                   "is listed. This method is eliminated."),
             immediate. = TRUE)
     opts[["spatialSign"]] <- NULL
-  }  
-  
+  }
+
   methods <- names(opts)
   ## check for inconsistent options for each predictor
   if(all(c("pca", "ica") %in% methods)){
@@ -907,10 +913,10 @@ pre_process_options <- function(opts, vars) {
                   paste("'", dup_imps, "'", sep = "", collapse = ", "))))
     }
   }
-  
+
   if(any(methods %in% "range") & any(methods %in% c("center", "scale", "BoxCox")))
     stop("Centering, scaling and/or Box-Cox transformations are inconsistent with scaling to a range")
-  
+
   ## coerce certain options based on others
   if("pca" %in% methods) {
     if("range" %in% methods) {
@@ -929,7 +935,7 @@ pre_process_options <- function(opts, vars) {
       opts[["center"]] <- c(opts[["center"]], opts[["ica"]])
       opts[["scale"]] <- c(opts[["scale"]], opts[["ica"]])
     }
-  }  
+  }
   if("spatialSign" %in% methods) {
     if("range" %in% methods) {
       opts[["range"]] <- c(opts[["range"]], opts[["spatialSign"]])
@@ -946,27 +952,27 @@ pre_process_options <- function(opts, vars) {
       opts[["center"]] <- num_vars
       opts[["scale"]] <- num_vars
     }
-  }  
+  }
   opts <- lapply(opts, unique)
-  
+
   ## check length of options and remove zero lengths
   opt_len <- unlist(lapply(opts, length))
   if(opt_len["spatialSign"] == 0 & any(unlist(wildcards) == "spatialSign"))
     opt_len["spatialSign"] <- 1
   if(any(opt_len < 1)) {
-    warning(paste("The following pre-processing methods were eliminated:", 
+    warning(paste("The following pre-processing methods were eliminated:",
                   paste("'", names(opts)[opt_len < 1], "'", sep = "", collapse = ", ")),
             immediate. = TRUE)
     opts <- opts[opt_len > 0]
   }
-  
+
   ## add to 'ignore'
   not_num_vars <- names(vars)[!vars]
-  if("ignore" %in% names(opts)) 
-    opts$ignore <- unique(c(not_num_vars, opts$ignore)) else 
+  if("ignore" %in% names(opts))
+    opts$ignore <- unique(c(not_num_vars, opts$ignore)) else
       opts$ignore <- not_num_vars
   ## TODO make sure that, if a var is in 'ignore' that it is nowhere else (and remove?)
-  
+
   list(opts = opts, wildcards = wildcards)
 }
 
@@ -981,7 +987,7 @@ get_types <- function(x, coarse = TRUE) {
       out <- unlist(lapply(x, function(x) class(x)[1]))
     }
   }
-  
+
   if(coarse) {
     num_classes <- c("integer", "numeric", "double")
     str_classes <- c("factor", "character")
@@ -998,7 +1004,7 @@ check_for_wildcards <- function(opts, verbose = TRUE){
   pc_wc <- unlist(lapply(opts, function(x) any(x == "_PC_")))
   if(any(pc_wc)) {
     pc_wc <- names(pc_wc)[pc_wc]
-    if(verbose) cat("PCA wildcards found for:", 
+    if(verbose) cat("PCA wildcards found for:",
                     paste(pc_wc, sep = "", collapse = ", "))
     if(any(pc_wc %in% other_methods)) {
       bad_ops <- pc_wc[pc_wc %in% other_methods]
@@ -1011,17 +1017,17 @@ check_for_wildcards <- function(opts, verbose = TRUE){
   ic_wc <- unlist(lapply(opts, function(x) any(x == "_IC_")))
   if(any(ic_wc)) {
     ic_wc <- names(ic_wc)[ic_wc]
-    if(verbose) cat("ICA wildcards found for:", 
+    if(verbose) cat("ICA wildcards found for:",
                     paste(ic_wc, sep = "", collapse = ", "), "\n")
     if(any(ic_wc %in% other_methods)) {
       bad_ops <- ic_wc[ic_wc %in% other_methods]
       if(verbose) cat(" ...but should not be in methods:",
                       paste(bad_ops, sep = "", collapse = ", "))
       for(i in bad_ops) opts[[i]] <- opts[[i]][opts[[i]] != "_IC_"]
-    } 
+    }
     if(verbose) cat("\n")
   }
-  
+
   pc_wc <- unlist(lapply(opts, function(x) any(x == "_PC_")))
   ic_wc <- unlist(lapply(opts, function(x) any(x == "_IC_")))
   wc_list <- list(PCA = names(pc_wc)[pc_wc],
@@ -1031,7 +1037,7 @@ check_for_wildcards <- function(opts, verbose = TRUE){
 }
 
 
-group_bc <- function(x, outcome = NULL, 
+group_bc <- function(x, outcome = NULL,
                      fudge,
                      na.remove,
                      numUnique,
@@ -1055,7 +1061,7 @@ group_bc <- function(x, outcome = NULL,
     if(verbose) cat("Box-Cox failed for:",  paste(bad_lambda, sep = "", collapse = ", "))
     bc <- bc[!(names(bc) %in% bad_lambda)]
   }
-  
+
   bc[!is.null(bc) & !is.na(bc)]
 }
 
@@ -1063,16 +1069,16 @@ convert_method <- function(x) {
   new_method <- list()
   if("center" %in% x$method)       new_method$center       <- names(x$mean)
   if("scale" %in% x$method)        new_method$scale        <- names(x$std)
-  if("YeoJohnson" %in% x$method)   new_method$YeoJohnson   <- names(x$yj)  
-  if("expoTrans" %in% x$method)    new_method$expoTrans    <- names(x$et)    
-  if("BoxCox" %in% x$method)       new_method$BoxCox       <- names(x$bc)     
+  if("YeoJohnson" %in% x$method)   new_method$YeoJohnson   <- names(x$yj)
+  if("expoTrans" %in% x$method)    new_method$expoTrans    <- names(x$et)
+  if("BoxCox" %in% x$method)       new_method$BoxCox       <- names(x$bc)
   if("knnImpute" %in% x$method)    new_method$knnImpute    <- names(x$mean)
-  if("bagImpute" %in% x$method)    new_method$bagImpute    <- names(x$bagImp) 
-  if("medianImpute" %in% x$method) new_method$medianImpute <- names(x$median)   
-  if("pca" %in% x$method)          new_method$pca          <- names(x$mean)  
-  if("ica" %in% x$method)          new_method$ica          <- names(x$mean)    
-  if("spatialSign" %in% x$method)  new_method$spatialSign  <- names(x$mean)   
-  if("invHyperbolicSine" %in% x$method)  new_method$invHyperbolicSine  <- x$method$invHyperbolicSine    
+  if("bagImpute" %in% x$method)    new_method$bagImpute    <- names(x$bagImp)
+  if("medianImpute" %in% x$method) new_method$medianImpute <- names(x$median)
+  if("pca" %in% x$method)          new_method$pca          <- names(x$mean)
+  if("ica" %in% x$method)          new_method$ica          <- names(x$mean)
+  if("spatialSign" %in% x$method)  new_method$spatialSign  <- names(x$mean)
+  if("invHyperbolicSine" %in% x$method)  new_method$invHyperbolicSine  <- x$method$invHyperbolicSine
   x$method <- new_method
   x
 }
